/**********************************************************************************************/
/*                                                                                            */
/*                                .                                    */
/*                                                                                            */
/**********************************************************************************************/
#include  "timer.h"

/*--------------------------------------------------------------------------------------------*/
/*   .                                                               */
/*--------------------------------------------------------------------------------------------*/
struct TIMER Timer[8];

/*--------------------------------------------------------------------------------------------*/
/*    .                                                */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Sys_Timer_Init(void)
{
  //   .
  for(register IDX i = 0; i < 8; i++) Timer[i].Flags = 0;
  //   1    m.
  TCCR3A = TCCR3B = TCCR3C = 0;
  OCR3A  = CPU_CLOCK_in_MHz / 1000;
  TCNT3  = 0;
  //           .
  ETIFR_OCF3A = 1;
  ETIMSK_OCIE3A = 1;
  TCCR3B_WGM32 = 1;
  TCCR3B |= 0x01;
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   .                                                                 */
/* :     (0 - 7),                                                        */
/*                   (1..65535),                                       */
/*                   ( NULL),                           */
/*                .                                                   */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
__monitor void Start_Timer(IN IDX timer, IN WORD interval , IN pTIMER_FUNC func, IN bool cycle)
{
  Timer[timer].Interval = interval;
  Timer[timer].Counter = interval;
  Timer[timer].pFunc = func;
  Timer[timer].Timer_Cycle = cycle;
  Timer[timer].Timer_Ena = TIMER_ENA;
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   .                                                               */
/* :     (0 - 7).                                                        */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
__monitor void Stop_Timer(IN IDX timer)
{
  Timer[timer].Timer_Ena = TIMER_DIS;
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*                                                                                            */
/*                  3   .           */
/*                                                                                            */
/*--------------------------------------------------------------------------------------------*/
#pragma vector = TIMER3_COMPA_vect
__interrupt void Timer3_CompA_Interrupt(void)
{
  //    .
  __enable_interrupt();
  //          .
  for(register IDX timer = 0; timer < 8; timer++)
     {
       if((Timer[timer].Timer_Ena == TIMER_ENA) && (--Timer[timer].Counter == 0))
         {
           //  . ,     .
           if(Timer[timer].pFunc == NULL)
             {
               //       -   .
               Timer[timer].Timer_Ena = TIMER_DIS;
             }
           else
             {
               //      .
               //     
               // (..        ).
               if(Timer[timer].Timer_Cycle == TIMER_CYCLE) Timer[timer].Counter = Timer[timer].Interval;
               else Timer[timer].Timer_Ena = TIMER_DIS;
               (*Timer[timer].pFunc)();
             }
         }
     }
  return;
}

/**********************************************************************************************/
